use std::env;
use std::process::exit;
use std::str::FromStr;

use getopts::Options;
use log::{info, SetLoggerError};

mod commands;
mod package;

fn main() {
    setup_logger()
        .expect("Unable to setup logger.");

    let args: Vec<String> = env::args().collect();
    let program = args[0].clone();

    let mut opts = Options::new();
    opts.optflag("h", "help", "Show help message.");
    let matches = opts.parse(&args[1..]).unwrap_or_else(|e| panic!("{}", e.to_string()));

    if matches.opt_present("h") {
        print_usage(&program, opts);
        exit(1);
    }

    let command = if !matches.free.is_empty() {
        matches.free[0].clone()
    } else {
        print_usage(&program, opts);
        exit(1);
    };

    info!("Identifier: {}", package::identifier::PackageIdentifier::from_str(&command).unwrap());
}

fn print_usage(program: &str, opts: Options) {
    let mut brief = format!("Usage: {} <COMMAND> [options]\n\n", program);
    brief += &*format!("Commands: grab \n");
    brief += &*format!("          remove\n");
    print!("{}", opts.usage(&brief.trim_end()));
}

fn setup_logger() -> Result<(), SetLoggerError> {
    fern::Dispatch::new()
        .format(|out, message, record| {
            out.finish(format_args!(
                "{} {}",
                match record.level().to_string().chars().nth(0).unwrap_or('I') {
                    'E' | 'W' => "!!",
                    _ => "**",
                },
                message
            ))
        })
        .level(log::LevelFilter::Info)
        .chain(std::io::stdout())
        .apply()
}